@@ -3,11 +3,13 @@ import warnings
 from types import ModuleType
 from typing import Any
 
+from zarr.errors import ZarrDeprecationWarning
 from zarr.storage._common import StoreLike, StorePath
 from zarr.storage._fsspec import FsspecStore
 from zarr.storage._local import LocalStore
 from zarr.storage._logging import LoggingStore
 from zarr.storage._memory import GpuMemoryStore, MemoryStore
+from zarr.storage._obstore import ObjectStore
 from zarr.storage._wrapper import WrapperStore
 from zarr.storage._zip import ZipStore
 
@@ -17,6 +19,7 @@ __all__ = [
     "LocalStore",
     "LoggingStore",
     "MemoryStore",
+    "ObjectStore",
     "StoreLike",
     "StorePath",
     "WrapperStore",
@@ -31,7 +34,7 @@ class VerboseModule(ModuleType):
                 "setting zarr.storage.default_compressor is deprecated, use "
                 "zarr.config to configure array.v2_default_compressor "
                 "e.g. config.set({'codecs.zstd':'numcodecs.Zstd', 'array.v2_default_compressor.numeric': 'zstd'})",
-                DeprecationWarning,
+                ZarrDeprecationWarning,
                 stacklevel=1,
             )
         else:
