@@ -8,9 +8,14 @@ from zarr.abc.store import Store
 from zarr.storage import LocalStore, ZipStore
 from zarr.testing.stateful import ZarrHierarchyStateMachine, ZarrStoreStateMachine
 
-pytestmark = pytest.mark.slow_hypothesis
+pytestmark = [
+    pytest.mark.slow_hypothesis,
+    # TODO: work out where this warning is coming from and fix
+    pytest.mark.filterwarnings("ignore:Unclosed client session:ResourceWarning"),
+]
 
 
+@pytest.mark.filterwarnings("ignore::zarr.core.dtype.common.UnstableSpecificationWarning")
 def test_zarr_hierarchy(sync_store: Store):
     def mk_test_instance_sync() -> ZarrHierarchyStateMachine:
         return ZarrHierarchyStateMachine(sync_store)
